@@ -41,19 +41,21 @@ int main()
 
     QtLua::State state;
     state.openlib(QtLua::QtLuaLib);
+    state.enable_qdebug_print(true);
 
     // Declare a lua global variable using our hash proxy
     state["hash"] = proxy;
 
     // Insert a value in QHash directly
-    hash.insert(QtLua::Value(state, "key1"), QtLua::Value(state, "value"));
+    hash.insert(QtLua::Value(&state, "key1"),
+		QtLua::Value(&state, "value"));
 
     // Read/Write in QHash from lua using the proxy object
     state.exec_statements("hash.key2 = hash.key1");
 
     // Read back value in QHash inserted from lua script
-    QtLua::Value value = hash.value(QtLua::Value(state, "key2"),
-				    QtLua::Value(state));
+    QtLua::Value value = hash.value(QtLua::Value(&state, "key2"),
+				    QtLua::Value(&state));
 
     std::cout << value.to_string().constData() << std::endl;
 
