@@ -39,11 +39,14 @@ int main()
 
     // Vector proxy which provides access to our QVector from lua.
     // Our proxy is allowed to resize the QVector.
-    QtLua::QVectorProxy<Container, true> proxy(vector);
+    QtLua::QVectorProxy<Container, 10> proxy(vector);
 							/* anchor 2 */
 
     QtLua::State state;
     state.openlib(QtLua::QtLuaLib);
+							/* anchor end */
+    state.enable_qdebug_print(true);
+							/* anchor 2 */
 
     // Declare a lua global variable using our QVector proxy
     state["vector"] = proxy;
