@@ -20,7 +20,6 @@
 
 
 #include <QtLua/Plugin>
-#include <QtLua/PluginInterface>
 #include <QtLua/String>
 
 #include "config.hh"
@@ -37,12 +36,18 @@ Plugin::Plugin(const String &filename)
   api<PluginInterface>()->register_members(*this);
 }
 
+Plugin::~Plugin()
+{
+  for (plugin_map_t::const_iterator i = _map.begin(); i != _map.end(); i++)
+    delete *i;
+}
+
 Plugin::Loader::Loader(const String &filename)
   : QPluginLoader(filename.to_qstring())
 {
   if (!load())
-    throw String("Unable to load plugin '%': %")
-      .arg(filename).arg(errorString());
+    QTLUA_THROW(Plugin::Loader, "Error loading plugin `%': %",
+		.arg(filename).arg(errorString()));
 }
 
 Plugin::Loader::~Loader()
@@ -50,21 +55,11 @@ Plugin::Loader::~Loader()
   unload();
 }
 
-Value Plugin::to_table(State &ls) const
+Value Plugin::to_table(State *ls) const
 {
   return Value(ls, _map);
 }
 
-const String & Plugin::get_name() const
-{
-  return api<PluginInterface>()->get_name();
-}
-
-String Plugin::get_value_str() const
-{
-  return get_name();
-}
-
 const String & Plugin::get_plugin_ext()
 {
   static const String s(STR(SHLIBEXT));
