@@ -23,6 +23,7 @@
 #include <cstdlib>
 
 #include <QStringList>
+#include <QDebug>
 
 #include <QtLua/State>
 #include <QtLua/UserData>
@@ -31,6 +32,7 @@
 #include <QtLua/Iterator>
 #include <QtLua/String>
 #include <QtLua/Function>
+#include <QtLua/Console>
 #include <internal/QObjectWrapper>
 
 #include "qtluaqtlib.hh"
@@ -43,35 +45,49 @@ extern "C" {
 
 namespace QtLua {
 
+char State::_key_threads;
 char State::_key_item_metatable;
 char State::_key_this;
 
+  /* save current thread lua_State and set new lua_State */
+#define QTLUA_SWITCH_THREAD(this_, st)	     \
+    lua_State *prev_th = this_->_lst; \
+    this_->_lst = st;
+
+  /* restore old thread State */
+#define QTLUA_RESTORE_THREAD(this_)     \
+    this_->_lst = prev_th;
+
 /************************************************************************
 	lua c functions
 ************************************************************************/
 
 int State::lua_cmd_iterator(lua_State *st)
 {
+  State		*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+
   try {
-    State		*this_ = get_this(st);
     Iterator::ptr	i = Value(1, this_).to_userdata_cast<Iterator>();
 
     if (i->more())
       {
-	i->get_key().push_value();
-	i->get_value().push_value();
+	i->get_key().push_value(st);
+	i->get_value().push_value(st);
 	i->next();
+	QTLUA_RESTORE_THREAD(this_);
 	return 2;
       }
     else
       {
 	lua_pushnil(st);
+	QTLUA_RESTORE_THREAD(this_);
 	return 1;
       }
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
   std::abort();
@@ -79,14 +95,19 @@ int State::lua_cmd_iterator(lua_State *st)
 
 int State::lua_cmd_each(lua_State *st)
 {
+  State               *this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+
   int idx = 1;
 
-  if (lua_gettop(st) < 1)
-    idx = LUA_GLOBALSINDEX;
-
   try {
-    State		*this_ = get_this(st);
-    Value		table(idx, this_);
+    Value		table;
+
+    if (lua_gettop(st) < 1)
+      table = Value::new_global_env(this_);
+    else
+      table = Value(idx, this_);
+
     Iterator::ptr	i = table.new_iterator();
 
     lua_pushcfunction(st, lua_cmd_iterator);
@@ -94,67 +115,77 @@ int State::lua_cmd_each(lua_State *st)
     lua_pushnil(st);
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
+  QTLUA_RESTORE_THREAD(this_);
   return 3;
 }
 
 int State::lua_cmd_print(lua_State *st)
 {
-  try {
-    State	*this_ = get_this(st);
+  State *this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
 
+  try {
     for (int i = 1; i <= lua_gettop(st); i++)
       {
 	String s = Value::to_string_p(st, i, true);
-	this_->output_str(s);
-	this_->output_str("\n");
-	qDebug("QtLua print:%s", s.constData());
+	if (this_->_debug_output)
+	  qDebug() << s; //"QtLua print:%s", s.constData());
+	this_->output_str(s + "\n");
       }
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
+  QTLUA_RESTORE_THREAD(this_);
   return 0;
 }
 
 int State::lua_cmd_plugin(lua_State *st)
 {
+  State	*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+
   try {
-    State	*this_ = get_this(st);
 
     if (lua_gettop(st) < 1 || !lua_isstring(st, 1))
       {
 	this_->output_str("Usage: plugin(\"library_filename_without_ext\")\n");
+	QTLUA_RESTORE_THREAD(this_);
 	return 0;
       }
 
     QTLUA_REFNEW(Plugin, String(lua_tostring(st, 1)) + Plugin::get_plugin_ext())->push_ud(st);
+    QTLUA_RESTORE_THREAD(this_);
     return 1;
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
-  return 0;
+  std::abort();
 }
 
 int State::lua_cmd_list(lua_State *st)
 {
+  State	*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+
   try {
-    State	*this_ = get_this(st);
+    Value		table;
 
-    int idx = lua_gettop(st) > 0 ? 1 : LUA_GLOBALSINDEX;
+    if (lua_gettop(st) < 1)
+      table = Value::new_global_env(this_);
+    else
+      table = Value(1, this_);
 
-    // display table object content
-    const Value	t = Value(idx, this_);
-
-    for (Value::const_iterator i = t.begin(); i != t.end(); i++)
+    for (Value::const_iterator i = table.begin(); i != table.end(); i++)
       {
 	try {
 	  this_->output_str(String("\033[18m") + i.value().type_name_u() + "\033[2m " +
@@ -166,20 +197,23 @@ int State::lua_cmd_list(lua_State *st)
       }
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
+  QTLUA_RESTORE_THREAD(this_);
   return 0;
 }
 
 int State::lua_cmd_help(lua_State *st)
 {
   State *this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
 
   if (lua_gettop(st) < 1)
     {
-      this_->output_str("Usage: help(function)\n");
+      this_->output_str("Usage: help(QtLua::Function object)\n");
+      QTLUA_RESTORE_THREAD(this_);
       return 0;
     }
 
@@ -192,14 +226,37 @@ int State::lua_cmd_help(lua_State *st)
       if (cmd.valid())
 	{
 	  this_->output_str(cmd->get_help() + "\n");
+	  QTLUA_RESTORE_THREAD(this_);
 	  return 0;
 	}
     }
 
   this_->output_str("Help is only available for QtLua::Function objects\n");
+
+  QTLUA_RESTORE_THREAD(this_);
   return 0;
 }
 
+int State::lua_cmd_qtype(lua_State *st)
+{
+  State *this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+
+  if (lua_gettop(st) < 1)
+    {
+      this_->output_str("Usage: qtype(value)\n");
+      QTLUA_RESTORE_THREAD(this_);
+      return 0;
+    }
+
+  Value v(1, this_);
+  String type(v.type_name_u());
+  lua_pushstring(st, type.constData());
+
+  QTLUA_RESTORE_THREAD(this_);
+  return 1;
+}
+
 // lua item metatable methods
 
 #define LUA_META_2OP_FUNC(n, op)					\
@@ -208,23 +265,25 @@ int State::lua_meta_item_##n(lua_State *st)				\
 {									\
   int		x = lua_gettop(st);					\
   State		*this_ = get_this(st);					\
+  QTLUA_SWITCH_THREAD(this_, st);					\
 									\
   try {									\
     Value	a(1, this_);						\
     Value	b(2, this_);						\
 									\
     if (a.type() == Value::TUserData)					\
-      a.to_userdata()->meta_operation(*this_, op, a, b).push_value();	\
+      a.to_userdata()->meta_operation(this_, op, a, b).push_value(st);	\
     else if (b.type() == Value::TUserData)				\
-      b.to_userdata()->meta_operation(*this_, op, a, b).push_value();	\
+      b.to_userdata()->meta_operation(this_, op, a, b).push_value(st);	\
     else								\
       std::abort();							\
 									\
   } catch (String &e) {							\
-    lua_pushstring(st, e.constData());					\
-    lua_error(st);							\
+    QTLUA_RESTORE_THREAD(this_);					\
+    luaL_error(st, "%s", e.constData());				\
   }									\
 									\
+  QTLUA_RESTORE_THREAD(this_);						\
   return lua_gettop(st) - x;						\
 }
 
@@ -234,17 +293,19 @@ int State::lua_meta_item_##n(lua_State *st)				\
 {									\
   int		x = lua_gettop(st);					\
   State		*this_ = get_this(st);					\
+  QTLUA_SWITCH_THREAD(this_, st);					\
 									\
   try {									\
     Value	a(1, this_);						\
 									\
-     a.to_userdata()->meta_operation(*this_, op, a, a).push_value();	\
+     a.to_userdata()->meta_operation(this_, op, a, a).push_value(st);	\
 									\
   } catch (String &e) {							\
-    lua_pushstring(st, e.constData());					\
-    lua_error(st);							\
+    QTLUA_RESTORE_THREAD(this_);					\
+    luaL_error(st, "%s", e.constData());				\
   }									\
 									\
+  QTLUA_RESTORE_THREAD(this_);						\
   return lua_gettop(st) - x;						\
 }
 
@@ -265,24 +326,25 @@ int State::lua_meta_item_index(lua_State *st)
 {
   int		x = lua_gettop(st);
   State		*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
 
   try {
     UserData::ptr ud = UserData::get_ud(st, 1);
 
     if (!ud.valid())
-      throw String("Can not index null lua::userdata value.");
+      QTLUA_THROW(QtLua::UserData, "Can not index a null `QtLua::UserData' value.");
 
     Value	op(2, this_);
 
-    Value v = ud->meta_index(*this_, op);
-    assert(v._st->_lst == st);
-    v.push_value();
+    Value v = ud->meta_index(this_, op);
+    v.push_value(st);
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
+  QTLUA_RESTORE_THREAD(this_);
   return lua_gettop(st) - x;
 }
 
@@ -290,23 +352,25 @@ int State::lua_meta_item_newindex(lua_State *st)
 {
   int		x = lua_gettop(st);
   State		*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
 
   try {
     UserData::ptr ud = UserData::get_ud(st, 1);
 
     if (!ud.valid())
-      throw String("Can not index null lua::userdata value.");
+      QTLUA_THROW(QtLua::UserData, "Can not index a null `QtLua::UserData' value.");
 
     Value	op1(2, this_);
     Value	op2(3, this_);
 
-    ud->meta_newindex(*this_, op1, op2);
+    ud->meta_newindex(this_, op1, op2);
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
+  QTLUA_RESTORE_THREAD(this_);
   return lua_gettop(st) - x;
 }
 
@@ -314,47 +378,160 @@ int State::lua_meta_item_call(lua_State *st)
 {
   int		n = lua_gettop(st);
   State		*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+  bool          yield = false;
 
   try {
     UserData::ptr ud = UserData::get_ud(st, 1);
 
     if (!ud.valid())
-      throw String("Can not call null lua::userdata value.");
+      QTLUA_THROW(QtLua::UserData, "Can not call a null `QtLua::UserData' value.");
 
     Value::List	args;
 
     for (int i = 2; i <= lua_gettop(st); i++)
       args.append(Value(i, this_));
 
-    args = ud->meta_call(*this_, args);
+    bool oy = this_->_yield_on_return;
+    this_->_yield_on_return = false;
+    args = ud->meta_call(this_, args);
+    yield = this_->_yield_on_return;
+    this_->_yield_on_return = oy;
 
     if (!lua_checkstack(st, args.size()))
-      throw String("Unable to extend lua stack to handle % return values").arg(args.size());
+      QTLUA_THROW(QtLua::State, "Unable to extend the lua stack to handle % return values",
+		  .arg(args.size()));
 
     foreach(const Value &v, args)
-      {
-	assert(v._st->_lst == st);
-	v.push_value();
-      }
+      v.push_value(st);
 
   } catch (String &e) {
-    lua_pushstring(st, e.constData());
-    lua_error(st);
+    QTLUA_RESTORE_THREAD(this_);
+    luaL_error(st, "%s", e.constData());
   }
 
-  return lua_gettop(st) - n;
+  QTLUA_RESTORE_THREAD(this_);
+  int nresults = lua_gettop(st) - n;
+  return yield ? lua_yield(st, nresults) : nresults;
 }
 
 int State::lua_meta_item_gc(lua_State *st)
 {
+  State		*this_ = get_this(st);
+  QTLUA_SWITCH_THREAD(this_, st);
+
   UserData::get_ud(st, 1).~Ref<UserData>();
 
+  QTLUA_RESTORE_THREAD(this_);
   return 0;
 }
 
 /************************************************************************/
 
-bool State::set_global_r(const String &name, const Value &value, int tblidx)
+static int lua_gettable_wrapper(lua_State *st)
+{
+  lua_gettable(st, 1);
+  return 1;
+}
+
+void State::lua_pgettable(lua_State *st, int index)
+{
+  if (lua_type(st, index) == LUA_TTABLE)
+    {
+      if (!lua_getmetatable(st, index))
+	return lua_rawget(st, index);
+      lua_pop(st, 1);
+    }
+
+  lua_pushcfunction(st, lua_gettable_wrapper);
+  if (index < 0
+#if LUA_VERSION_NUM < 502
+      && index != LUA_GLOBALSINDEX
+#endif
+      )
+    index--;
+  lua_pushvalue(st, index);  // table
+  lua_pushvalue(st, -3);     // key
+  if (lua_pcall(st, 2, 1, 0))
+    {
+      String err(lua_tostring(st, -1));
+      lua_pop(st, 1);
+      throw err;
+    }
+  lua_remove(st, -2);     // replace key by value
+}
+
+static int lua_settable_wrapper(lua_State *st)
+{
+  lua_settable(st, 1);
+  return 0;
+}
+
+void State::lua_psettable(lua_State *st, int index)
+{
+  if (lua_type(st, index) == LUA_TTABLE)
+    {
+      if (!lua_getmetatable(st, index))
+	return lua_rawset(st, index);
+      lua_pop(st, 1);
+    }
+
+  lua_pushcfunction(st, lua_settable_wrapper);
+  if (index < 0
+#if LUA_VERSION_NUM < 502
+      && index != LUA_GLOBALSINDEX
+#endif
+      )
+    index--;
+  lua_pushvalue(st, index);  // table
+  lua_pushvalue(st, -4);     // key
+  lua_pushvalue(st, -4);     // value
+  if (lua_pcall(st, 3, 0, 0))
+    {
+      String err(lua_tostring(st, -1));
+      lua_pop(st, 1);
+      throw err;
+    }
+  lua_pop(st, 2);     // remove key/value
+}
+
+static int lua_next_wrapper(lua_State *st)
+{
+  return lua_next(st, 1) ? 2 : 0;
+}
+
+/** Protected lua_next, same behavior as lua_next. On error this
+    function throw an exception and leave the lua stack untouched (the
+    key is not poped). */
+int State::lua_pnext(lua_State *st, int index)
+{
+  lua_pushcfunction(st, lua_next_wrapper);
+  if (index < 0
+#if LUA_VERSION_NUM < 502
+      && index != LUA_GLOBALSINDEX
+#endif
+      )
+    index--;
+  lua_pushvalue(st, index);  // table
+  lua_pushvalue(st, -3);     // key
+  if (lua_pcall(st, 2, 2, 0))
+    {
+      String err(lua_tostring(st, -1));
+      lua_pop(st, 1);
+      throw err;
+    }
+  lua_remove(st, -3);	     // remove key
+  if (lua_isnil(st, -2))
+    {
+      lua_pop(st, 2);
+      return 0;
+    }
+  return 1;
+}
+
+/************************************************************************/
+
+void State::set_global_r(const String &name, const Value &value, int tblidx)
 {
   int len = name.indexOf('.', 0);
 
@@ -362,10 +539,20 @@ bool State::set_global_r(const String &name, const Value &value, int tblidx)
     {
       // set value in table if last
       lua_pushstring(_lst, name.constData());
-      value.push_value();
-      lua_settable(_lst, tblidx);
 
-      return true;
+      try {
+	value.push_value(_lst);
+      } catch (...) {
+	lua_pop(_lst, 1);
+	throw;
+      }
+
+      try {
+	lua_psettable(_lst, tblidx);
+      } catch (...) {
+	lua_pop(_lst, 2);
+	throw;
+      }
     }
   else
     {
@@ -373,7 +560,13 @@ bool State::set_global_r(const String &name, const Value &value, int tblidx)
       String prefix(name.mid(0, len));
 
       lua_pushstring(_lst, prefix.constData());
-      lua_gettable(_lst, tblidx);
+
+      try {
+	lua_pgettable(_lst, tblidx);
+      } catch (...) {
+	lua_pop(_lst, 1);
+	throw;
+      }
 
       if (lua_isnil(_lst, -1))
 	{
@@ -382,37 +575,48 @@ bool State::set_global_r(const String &name, const Value &value, int tblidx)
 	  lua_pushstring(_lst, prefix.constData());
 	  lua_newtable(_lst);
 
-	  if (set_global_r(name.mid(len + 1), value, lua_gettop(_lst)))
-	    {
-	      lua_settable(_lst, tblidx);
-	      return true;
-	    }
-	  else
-	    {
-	      lua_pop(_lst, 2);
-	      return false;
-	    }
+	  try {
+	    set_global_r(name.mid(len + 1), value, lua_gettop(_lst));
+	    lua_psettable(_lst, tblidx);
+	  } catch (...) {
+	    lua_pop(_lst, 2);
+	    throw;
+	  }
 	}
       else if (lua_istable(_lst, -1))
 	{
 	  // use existing intermediate table
-	  bool res = set_global_r(name.mid(len + 1), value, lua_gettop(_lst));
-	  lua_pop(_lst, 1);
-	  return res;
+	  try {
+	    set_global_r(name.mid(len + 1), value, lua_gettop(_lst));
+	    lua_pop(_lst, 1);
+	  } catch (...) {
+	    lua_pop(_lst, 1);
+	    throw;
+	  }
 	}
       else
 	{
 	  // bad existing intermediate value
 	  lua_pop(_lst, 1);
-	  return false;
+	  QTLUA_THROW(QtLua::State, "Can not set the global, the `%' key already exists.", .arg(name));
 	}
     }
 }
 
 void State::set_global(const String &name, const Value &value)
 {
-  if (!set_global_r(name, value, LUA_GLOBALSINDEX))
-    throw String("Unable to set lua global variable.");
+#if LUA_VERSION_NUM < 502
+  set_global_r(name, value, LUA_GLOBALSINDEX);
+#else
+  try {
+    lua_pushglobaltable(_lst);
+    set_global_r(name, value, lua_gettop(_lst));
+    lua_pop(_lst, 1);
+  } catch (...) {
+    lua_pop(_lst, 1);
+    throw;
+  }
+#endif
 }
 
 void State::get_global_r(const String &name, Value &value, int tblidx) const
@@ -423,7 +627,14 @@ void State::get_global_r(const String &name, Value &value, int tblidx) const
     {
       // get value from table if last
       lua_pushstring(_lst, name.constData());
-      lua_gettable(_lst, tblidx);
+
+      try {
+	lua_pgettable(_lst, tblidx);
+      } catch (...) {
+	lua_pop(_lst, 1);
+	throw;
+      }
+
       value = Value(-1, this);
       lua_pop(_lst, 1);
     }
@@ -433,37 +644,96 @@ void State::get_global_r(const String &name, Value &value, int tblidx) const
       String prefix(name.mid(0, len));
 
       lua_pushstring(_lst, prefix.constData());
-      lua_gettable(_lst, tblidx);
 
-      if (lua_istable(_lst, -1))
+      try {
+	lua_pgettable(_lst, tblidx);
+      } catch (...) {
+	lua_pop(_lst, 1);
+	throw;
+      }
+
+      if (!lua_istable(_lst, -1))
 	{
-	  get_global_r(name.mid(len + 1), value, lua_gettop(_lst));
 	  lua_pop(_lst, 1);
+	  QTLUA_THROW(QtLua::State, "Can not get the global, `%' is not a table.", .arg(prefix));
 	}
 
+      try {
+	get_global_r(name.mid(len + 1), value, lua_gettop(_lst));
+      } catch (...) {
+	lua_pop(_lst, 1);
+	throw;
+      }
+
       lua_pop(_lst, 1);
     }
 }
 
 Value State::get_global(const String &path) const
 {
-  Value res(const_cast<State&>(*this));
+  Value res(const_cast<State*>(this));
+
+#if LUA_VERSION_NUM < 502
   get_global_r(path, res, LUA_GLOBALSINDEX);
+#else
+  try {
+    lua_pushglobaltable(_lst);
+    get_global_r(path, res, lua_gettop(_lst));
+    lua_pop(_lst, 1);
+  } catch (...) {
+    lua_pop(_lst, 1);
+    throw;
+  }
+#endif
+
   return res;
 }
 
-Value State::operator[] (const Value &key) const
+Value State::at(const Value &key) const
 {
-  key.push_value();
-  lua_gettable(_lst, LUA_GLOBALSINDEX);
+#if LUA_VERSION_NUM < 502
+  key.push_value(_lst);
+  try {
+    lua_pgettable(_lst, LUA_GLOBALSINDEX);
+  } catch (...) {
+    lua_pop(_lst, 1);
+    throw;
+  }
+
   Value res(-1, this);
   lua_pop(_lst, 1);
+#else
+
+  try {
+    lua_pushglobaltable(_lst);
+    key.push_value(_lst);
+  } catch (...) {
+    lua_pop(_lst, 1);
+    throw;
+  }
+
+  try {
+    lua_pgettable(_lst, -2);
+  } catch (...) {
+    lua_pop(_lst, 2);
+    throw;
+  }
+
+  Value res(-1, this);
+  lua_pop(_lst, 2);
+#endif
+
   return res;
 }
 
 ValueRef State::operator[] (const Value &key)
 {
-  return ValueRef(Value(LUA_GLOBALSINDEX, this), key);
+  return ValueRef(Value::new_global_env(this), key);
+}
+
+void State::check_empty_stack() const
+{
+  assert(!lua_gettop(_lst));
 }
 
 State::State()
@@ -476,23 +746,26 @@ State::State()
   assert(Value::TTable == LUA_TTABLE);
   assert(Value::TFunction == LUA_TFUNCTION);
   assert(Value::TUserData == LUA_TUSERDATA);
+  assert(Value::TThread == LUA_TTHREAD);
 
-  _lst = lua_open();
+#if LUA_VERSION_NUM < 501
+  _mst = _lst = lua_open();
+#else
+  _mst = _lst = luaL_newstate();
+#endif
 
-  if (!_lst)
+  if (!_mst)
     throw std::bad_alloc();
 
-  //lua_atpanic(_lst, lua_panic);
-
   // creat metatable for UserData events
 
-  lua_pushlightuserdata(_lst, &_key_item_metatable);
-  lua_newtable(_lst);
+  lua_pushlightuserdata(_mst, &_key_item_metatable);
+  lua_newtable(_mst);
 
 #define LUA_META_BIND(n)			\
-  lua_pushstring(_lst, "__" #n);			\
-  lua_pushcfunction(_lst, lua_meta_item_##n);	\
-  lua_rawset(_lst, -3);
+  lua_pushstring(_mst, "__" #n);			\
+  lua_pushcfunction(_mst, lua_meta_item_##n);	\
+  lua_rawset(_mst, -3);
 
   LUA_META_BIND(add);
   LUA_META_BIND(sub);
@@ -511,13 +784,30 @@ State::State()
   LUA_META_BIND(call);
   LUA_META_BIND(gc);
 
-  lua_rawset(_lst, LUA_REGISTRYINDEX);
+  lua_rawset(_mst, LUA_REGISTRYINDEX);
 
   // pointer to this
 
-  lua_pushlightuserdata(_lst, &_key_this);
-  lua_pushlightuserdata(_lst, this);
-  lua_rawset(_lst, LUA_REGISTRYINDEX);
+  lua_pushlightuserdata(_mst, &_key_this);
+  lua_pushlightuserdata(_mst, this);
+  lua_rawset(_mst, LUA_REGISTRYINDEX);
+
+#if LUA_VERSION_NUM < 501
+  // create a weak table for threads, substitute for lua_pushthread
+  lua_pushlightuserdata(_mst, &_key_threads);
+  lua_newtable(_mst);
+
+  lua_newtable(_mst);    // metatable for weak table mode
+  lua_pushstring(_mst, "__mode");
+  lua_pushstring(_mst, "v");
+  lua_rawset(_mst, -3);
+  lua_setmetatable(_mst, -2);
+
+  lua_rawset(_mst, LUA_REGISTRYINDEX);
+#endif
+
+  _debug_output = false;
+  _yield_on_return = false;
 }
 
 State::~State()
@@ -527,7 +817,7 @@ State::~State()
     w->_lua_disconnect_all();
 
   // lua state close
-  lua_close(_lst);
+  lua_close(_mst);
 
   // wipe QObjectWrapper objects
   wrapper_hash_t::const_iterator i;
@@ -536,13 +826,6 @@ State::~State()
     i.value()->_drop();
 }
 
-int State::lua_panic(lua_State *st)
-{
-  String err(lua_tostring(st, -1));
-  lua_pop(st, 1);
-  throw err;
-}
-
 Value State::eval_expr(bool use_lua, const String &expr)
 {
   // Use lua to transform user input to lua value
@@ -551,7 +834,7 @@ Value State::eval_expr(bool use_lua, const String &expr)
       Value::List res = exec_statements(String("return ") + expr);
 
       if (res.empty())
-	throw String("lua expression `%' returned no value").arg(expr);
+	QTLUA_THROW(QtLua::State, "The lua expression `%' returned no value.", .arg(expr));
 
       return res[0];
     }
@@ -563,14 +846,14 @@ Value State::eval_expr(bool use_lua, const String &expr)
 	double number = expr.toDouble(&ok);
 
 	if (ok)
-	  return Value(*this, number);
+	  return Value(this, number);
 	else
 	  {
-	    // exprip double quotes if any
+	    // strip double quotes if any
 	    if (expr.size() > 1 && expr.startsWith('"') && expr.endsWith('"'))
-	      return Value(*this, String(expr.mid(1, expr.size() - 2)));
+	      return Value(this, String(expr.mid(1, expr.size() - 2)));
 	    else
-	      return Value(*this, expr);
+	      return Value(this, expr);
 	  }
       }
 }
@@ -595,7 +878,11 @@ Value::List State::exec_chunk(QIODevice &io)
   struct lua_reader_state_s rst;
   rst._io = &io;
 
+#if LUA_VERSION_NUM < 502
   if (lua_load(_lst, &lua_reader, &rst, ""))
+#else
+  if (lua_load(_lst, &lua_reader, &rst, "", NULL))
+#endif
     {
       String err(lua_tostring(_lst, -1));
       lua_pop(_lst, 1);
@@ -668,9 +955,8 @@ void State::gc_collect()
 
 void State::reg_c_function(const char *name, lua_CFunction f)
 {
-  lua_pushstring(_lst, name);
   lua_pushcfunction(_lst, f);
-  lua_rawset(_lst, LUA_GLOBALSINDEX);
+  lua_setglobal(_lst, name);
 }
 
 State * State::get_this(lua_State *st)
@@ -685,67 +971,123 @@ State * State::get_this(lua_State *st)
   return static_cast<State*>(data);
 }
 
-#define QTLUA_LUA_CALL(st, f)		\
+#if LUA_VERSION_NUM < 502
+# define QTLUA_LUA_CALL(st, f, modname)	\
   lua_pushcfunction(st, f);		\
   lua_call(st, 0, 0);
+#else
+# define QTLUA_LUA_CALL(st, f, modname)	\
+  luaL_requiref(st, modname, f, 1);     \
+  lua_pop(st, 1);
+#endif
 
-void State::openlib(Library lib)
+bool State::openlib(Library lib)
 {
   switch (lib)
     {
+    case CoroutineLib:
+#if LUA_VERSION_NUM >= 502
+      QTLUA_LUA_CALL(_lst, luaopen_coroutine, "coroutine");
+      return true;
+#endif
     case BaseLib:
-      QTLUA_LUA_CALL(_lst, luaopen_base);
-      return;
+      QTLUA_LUA_CALL(_lst, luaopen_base, "_G");
+      return true;
+#ifdef HAVE_LUA_PACKAGELIB
     case PackageLib:
-#ifdef HAVE_LUA_PACKAGELIB
-      QTLUA_LUA_CALL(_lst, luaopen_package);
+      QTLUA_LUA_CALL(_lst, luaopen_package, "package");
+      return true;
 #endif
-      return;
     case StringLib:
-      QTLUA_LUA_CALL(_lst, luaopen_string);
-      return;
+      QTLUA_LUA_CALL(_lst, luaopen_string, "string");
+      return true;
     case TableLib:
-      QTLUA_LUA_CALL(_lst, luaopen_table);
-      return;
+      QTLUA_LUA_CALL(_lst, luaopen_table, "table");
+      return true;
     case MathLib:
-      QTLUA_LUA_CALL(_lst, luaopen_math);
-      return;
+      QTLUA_LUA_CALL(_lst, luaopen_math, "math");
+      return true;
     case IoLib:
-      QTLUA_LUA_CALL(_lst, luaopen_io);
-      return;
+      QTLUA_LUA_CALL(_lst, luaopen_io, "io");
+      return true;
 #ifdef HAVE_LUA_OSLIB
     case OsLib:
-      QTLUA_LUA_CALL(_lst, luaopen_os);
-      return;
+      QTLUA_LUA_CALL(_lst, luaopen_os, "os");
+      return true;
 #endif
     case DebugLib:
-      QTLUA_LUA_CALL(_lst, luaopen_debug);
-      return;
-    case AllLibs:
-#ifdef HAVE_LUA_OPENLIBS
-      luaL_openlibs(_lst);
-#else
-      QTLUA_LUA_CALL(_lst, luaopen_base);
-      QTLUA_LUA_CALL(_lst, luaopen_string);
-      QTLUA_LUA_CALL(_lst, luaopen_table);
-      QTLUA_LUA_CALL(_lst, luaopen_math);
-      QTLUA_LUA_CALL(_lst, luaopen_io);
-      QTLUA_LUA_CALL(_lst, luaopen_debug);
+      QTLUA_LUA_CALL(_lst, luaopen_debug, "debug");
+      return true;
+
+#if LUA_VERSION_NUM >= 502
+    case Bit32Lib:
+      QTLUA_LUA_CALL(_lst, luaopen_bit32, "bit32");
+      return true;
 #endif
-      qtluaopen_qt(*this);
+
+#ifdef HAVE_LUA_JITLIB
+    case JitLib:
+      QTLUA_LUA_CALL(_lst, luaopen_jit, "jit");
+      return true;
+#endif
+#ifdef HAVE_LUA_FFILIB
+    case FfiLib:
+      QTLUA_LUA_CALL(_lst, luaopen_ffi, "ffi");
+      return true;
+#endif
+
+    case AllLibs:
+#if LUA_VERSION_NUM >= 502
+      QTLUA_LUA_CALL(_lst, luaopen_coroutine, "coroutine");
+      QTLUA_LUA_CALL(_lst, luaopen_bit32, "bit32");
+#endif
+#ifdef HAVE_LUA_OSLIB
+      QTLUA_LUA_CALL(_lst, luaopen_os, "os");
+#endif
+#ifdef HAVE_LUA_PACKAGELIB
+      QTLUA_LUA_CALL(_lst, luaopen_package, "package");
+#endif
+      QTLUA_LUA_CALL(_lst, luaopen_base, "_G");
+      QTLUA_LUA_CALL(_lst, luaopen_string, "string");
+      QTLUA_LUA_CALL(_lst, luaopen_table, "table");
+      QTLUA_LUA_CALL(_lst, luaopen_math, "math");
+      QTLUA_LUA_CALL(_lst, luaopen_io, "io");
+      QTLUA_LUA_CALL(_lst, luaopen_debug, "debug");
+#ifdef HAVE_LUA_JITLIB
+      QTLUA_LUA_CALL(_lst, luaopen_jit, "jit");
+#endif
+#ifdef HAVE_LUA_FFILIB
+      QTLUA_LUA_CALL(_lst, luaopen_ffi, "ffi");
+#endif
+      qtluaopen_qt(this);
+
     case QtLuaLib:
       reg_c_function("print", lua_cmd_print);
       reg_c_function("list", lua_cmd_list);
       reg_c_function("each", lua_cmd_each);
       reg_c_function("help", lua_cmd_help);
       reg_c_function("plugin", lua_cmd_plugin);
-      return;
+      reg_c_function("qtype", lua_cmd_qtype);
+      return true;
+
     case QtLib:
-      qtluaopen_qt(*this);
-      return;
+      qtluaopen_qt(this);
+      return true;
+
+    default:
+      return false;
     }
 }
 
+int State::lua_version() const
+{
+#if LUA_VERSION_NUM < 501
+  return 500;
+#else
+  return LUA_VERSION_NUM;
+#endif
+}
+
 void State::lua_do(void (*func)(lua_State *st))
 {
   func(_lst);
@@ -766,6 +1108,9 @@ void State::fill_completion_list_r(String &path, const String &prefix,
 	{
 	  const Value &k = i.key();
 
+	  if (list.size() >= QTLUA_MAX_COMPLETION)
+	    return;
+
 	  // ignore non string keys
 	  if (k.type() != Value::TString)
 	    continue;
@@ -782,20 +1127,28 @@ void State::fill_completion_list_r(String &path, const String &prefix,
 		  {
 		  case Value::TTable:
 
-		    v.push_value();
-		    lua_pushnil(_lst);
-		    if (lua_next(_lst, -2))
-		      {
-			if (lua_type(_lst, -2) == Value::TString)
-			  entry += ".";
-			else
+		    try {
+		      v.push_value(_lst);
+		      try {
+			lua_pushnil(_lst);
+			if (lua_pnext(_lst, -2))
 			  {
-			    entry += "[]";
-			    cursor_offset = -1;
+			    if (lua_type(_lst, -2) == Value::TString)
+			      entry += ".";
+			    else
+			      {
+				entry += "[]";
+				cursor_offset = -1;
+			      }
+			    lua_pop(_lst, 2);  // pop key/value
 			  }
-			lua_pop(_lst, 2);
+		      } catch (...) {
+			lua_pop(_lst, 1);  // pop key on pnext error
 		      }
-		    lua_pop(_lst, 1);
+		      lua_pop(_lst, 1);    // pop table
+		    } catch (...) {
+		    }
+		    check_empty_stack();
 
 		    break;
 
@@ -832,7 +1185,8 @@ void State::fill_completion_list_r(String &path, const String &prefix,
 	path += next;
 	if (len < prefix.size())
 	  path += prefix[len];
-	fill_completion_list_r(path, prefix.mid(len + 1), list, tbl[next], cursor_offset);
+	fill_completion_list_r(path, prefix.mid(len + 1), list,
+			       tbl.at(next), cursor_offset);
       } catch (...) {
       }
 
@@ -843,7 +1197,7 @@ void State::fill_completion_list(const QString &prefix, QStringList &list, int &
 {
   String path;
 
-  fill_completion_list_r(path, prefix, list, Value(LUA_GLOBALSINDEX, this), cursor_offset);
+  fill_completion_list_r(path, prefix, list, Value::new_global_env(this), cursor_offset);
 }
 
 }
